from setuptools import setup, find_packages

setup(
    name='disrpt_utils',
    version='0.5.0',
    packages=find_packages(),
    install_requires=[],
    author='Damien Sileo',
    description='disrpt utils',
    url='https://github.com/sileod/disrpt_utils/',
)